#
# Configuration file for using the XML library in GNOME applications
#
prefix="/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla"
exec_prefix="${prefix}"
libdir="${exec_prefix}/lib"
includedir="${prefix}/include"

XMLSEC_LIBDIR="${exec_prefix}/lib"
XMLSEC_INCLUDEDIR=" -D__XMLSEC_FUNCTION__=__func__ -DXMLSEC_NO_SIZE_T -DXMLSEC_NO_GOST=1 -DXMLSEC_NO_GOST2012=1 -DXMLSEC_DL_LIBLTDL=1 -I${prefix}/include/xmlsec1   -I/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla/include/libxml2 -I/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla/include -I/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla/include/libxml2 -I/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla/include  -I/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla/include -I/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla/include/openssl -DXMLSEC_CRYPTO_OPENSSL=1"
XMLSEC_LIBS="-L${exec_prefix}/lib -lxmlsec1-openssl -lxmlsec1 -lltdl  -lxml2 -L/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla/lib -lxslt -lxml2  -L/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_30bl4dbma9/croot/libxmlsec1_1688663233386/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pla/lib -lcrypto "
MODULE_VERSION="xmlsec-1.2.38-openssl"

